@@ -45,8 +45,6 @@ typedef gboolean (* RowCompareFunc) (GtkTreeModel *model,
 static GtkWidget *treeview;
 static gboolean   expanded;
 
-static GUPnPDIDLLiteParser *didl_parser;
-
 typedef struct
 {
   GUPnPServiceProxy *content_dir;
@@ -147,7 +145,7 @@ create_playlist_treemodel (void)
 {
         GtkTreeStore *store;
 
-        store = gtk_tree_store_new (6,
+        store = gtk_tree_store_new (7,
                                     /* Icon */
                                     GDK_TYPE_PIXBUF,
                                     /* Title */
@@ -159,7 +157,9 @@ create_playlist_treemodel (void)
                                     /* Id */
                                     G_TYPE_STRING,
                                     /* Is container? */
-                                    G_TYPE_BOOLEAN);
+                                    G_TYPE_BOOLEAN,
+                                    /* childCount */
+                                    G_TYPE_UINT);
 
         return GTK_TREE_MODEL (store);
 }
@@ -220,14 +220,16 @@ on_playlist_row_expanded (GtkTreeView *tree_view,
                 GUPnPServiceProxy *content_dir;
                 gchar             *id;
                 gboolean           is_container;
+                guint              child_count;
 
                 gtk_tree_model_get (model, &child_iter,
                                     3, &content_dir,
                                     4, &id,
                                     5, &is_container,
+                                    6, &child_count,
                                     -1);
 
-                if (is_container) {
+                if (is_container && child_count > 0) {
                         browse (content_dir, id, 0, MAX_BROWSE);
                 }
 
@@ -330,9 +332,6 @@ setup_playlist_treeview (GtkBuilder *builder)
                                                 NULL);
 
         expanded = FALSE;
-
-        didl_parser = gupnp_didl_lite_parser_new ();
-        g_assert (didl_parser != NULL);
 }
 
 static gboolean
@@ -442,12 +441,12 @@ compare_container (GtkTreeModel *model,
 }
 
 static GdkPixbuf *
-get_item_icon (xmlNode *object_node)
+get_item_icon (GUPnPDIDLLiteObject *object)
 {
-        GdkPixbuf *icon;
-        char      *class_name;
+        GdkPixbuf  *icon;
+        const char *class_name;
 
-        class_name = gupnp_didl_lite_object_get_upnp_class (object_node);
+        class_name = gupnp_didl_lite_object_get_upnp_class (object);
         if (G_UNLIKELY (class_name == NULL)) {
                 return get_icon_by_id (ICON_FILE);
         }
@@ -472,8 +471,6 @@ get_item_icon (xmlNode *object_node)
                 icon = get_icon_by_id (ICON_FILE);
         }
 
-        g_free (class_name);
-
         return icon;
 }
 
@@ -518,19 +515,45 @@ update_container (GUPnPServiceProxy *content_dir,
         model = gtk_tree_view_get_model (GTK_TREE_VIEW (treeview));
         g_assert (model != NULL);
 
-        if (!find_container (content_dir,
-                             model,
-                             &container_iter,
-                             container_id)) {
-                return;
-        } else if (gtk_tree_model_iter_has_child (model, &container_iter)) {
+        if (find_container (content_dir,
+                            model,
+                            &container_iter,
+                            container_id)) {
                 /* Remove everyting under the container */
-                unpopulate_container (model, &container_iter);
+                if (gtk_tree_model_iter_has_child (model, &container_iter)) {
+                        unpopulate_container (model, &container_iter);
+                }
+
                 /* Browse it again */
                 browse (content_dir, container_id, 0, MAX_BROWSE);
         }
 }
 
+static void
+update_container_child_count (GUPnPServiceProxy *content_dir,
+                              const char        *container_id)
+{
+        GtkTreeModel *model;
+        GtkTreeIter   container_iter;
+
+        model = gtk_tree_view_get_model (GTK_TREE_VIEW (treeview));
+        g_assert (model != NULL);
+
+        if (find_container (content_dir,
+                            model,
+                            &container_iter,
+                            container_id)) {
+                int child_count;
+
+                child_count = gtk_tree_model_iter_n_children (model,
+                                                              &container_iter);
+                gtk_tree_store_set (GTK_TREE_STORE (model),
+                                    &container_iter,
+                                    6, child_count,
+                                    -1);
+        }
+}
+
 static void
 on_container_update_ids (GUPnPServiceProxy *content_dir,
                          const char        *variable,
@@ -551,45 +574,40 @@ on_container_update_ids (GUPnPServiceProxy *content_dir,
 }
 
 static void
-append_didl_object (xmlNode      *object_node,
-                    BrowseData   *browse_data,
-                    GtkTreeModel *model,
-                    GtkTreeIter  *server_iter)
+append_didl_object (GUPnPDIDLLiteObject *object,
+                    BrowseData          *browse_data,
+                    GtkTreeModel        *model,
+                    GtkTreeIter         *server_iter)
 {
         GtkTreeIter parent_iter;
-        char       *id;
-        char       *parent_id;
-        char       *title;
+        const char *id;
+        const char *parent_id;
+        const char *title;
         gboolean    is_container;
+        uint        child_count;
         GdkPixbuf  *icon;
         gint        position;
 
-        id = gupnp_didl_lite_object_get_id (object_node);
-        if (id == NULL)
+        id = gupnp_didl_lite_object_get_id (object);
+        title = gupnp_didl_lite_object_get_title (object);
+        parent_id = gupnp_didl_lite_object_get_parent_id (object);
+        if (id == NULL || title == NULL || parent_id == NULL)
                 return;
 
-        title = gupnp_didl_lite_object_get_title (object_node);
-        if (title == NULL) {
-                g_free (id);
-                return;
-        }
-
-        parent_id = gupnp_didl_lite_object_get_parent_id (object_node);
-        if (parent_id == NULL) {
-                g_free (id);
-                g_free (title);
-                return;
-        }
-
-        is_container = gupnp_didl_lite_object_is_container (object_node);
+        is_container = GUPNP_IS_DIDL_LITE_CONTAINER (object);
 
         if (is_container) {
+                GUPnPDIDLLiteContainer *container;
                 position = 0;
                 icon = get_icon_by_id (ICON_CONTAINER);
+
+                container = GUPNP_DIDL_LITE_CONTAINER (object);
+                child_count = gupnp_didl_lite_container_get_child_count
+                                                                (container);
         } else {
                 position = -1;
-
-                icon = get_item_icon (object_node);
+                child_count = 0;
+                icon = get_item_icon (object);
         }
 
         /* Check if we browsed the root container. */
@@ -600,9 +618,8 @@ append_didl_object (xmlNode      *object_node,
                               &parent_iter,
                               compare_container,
                               (gpointer) parent_id,
-                              TRUE)) {
-                goto return_point;
-        }
+                              TRUE))
+                return;
 
         gtk_tree_store_insert_with_values (GTK_TREE_STORE (model),
                                            NULL, &parent_iter, position,
@@ -611,12 +628,8 @@ append_didl_object (xmlNode      *object_node,
                                            3, browse_data->content_dir,
                                            4, id,
                                            5, is_container,
+                                           6, child_count,
                                            -1);
-
-return_point:
-        g_free (parent_id);
-        g_free (title);
-        g_free (id);
 }
 
 static void
@@ -661,8 +674,8 @@ get_content_dir (GUPnPDeviceProxy *proxy)
 }
 
 static void
-on_didl_object_available (GUPnPDIDLLiteParser *didl_parser,
-                          xmlNode             *object_node,
+on_didl_object_available (GUPnPDIDLLiteParser *parser,
+                          GUPnPDIDLLiteObject *object,
                           gpointer             user_data)
 {
         BrowseData   *browse_data;
@@ -675,7 +688,8 @@ on_didl_object_available (GUPnPDIDLLiteParser *didl_parser,
 
         browse_data = (BrowseData *) user_data;
 
-        udn = gupnp_service_info_get_udn (GUPNP_SERVICE_INFO (browse_data->content_dir));
+        udn = gupnp_service_info_get_udn
+                                (GUPNP_SERVICE_INFO (browse_data->content_dir));
 
         if (find_row (model,
                        NULL,
@@ -683,7 +697,7 @@ on_didl_object_available (GUPnPDIDLLiteParser *didl_parser,
                        compare_media_server,
                        (gpointer) udn,
                        FALSE)) {
-                append_didl_object (object_node,
+                append_didl_object (object,
                                     browse_data,
                                     model,
                                     &server_iter);
@@ -722,20 +736,26 @@ browse_cb (GUPnPServiceProxy       *content_dir,
                                         &total_matches,
                                         NULL);
         if (didl_xml) {
-                guint32 remaining;
-                GError *error;
+                GUPnPDIDLLiteParser *parser;
+                guint32              remaining;
+                GError              *error;
 
                 error = NULL;
-                if (!gupnp_didl_lite_parser_parse_didl
-                                                (didl_parser,
-                                                 didl_xml,
-                                                 on_didl_object_available,
-                                                 data,
-                                                 &error)) {
+                parser = gupnp_didl_lite_parser_new ();
+
+                g_signal_connect (parser,
+                                  "object-available",
+                                  G_CALLBACK (on_didl_object_available),
+                                  data);
+
+                if (!gupnp_didl_lite_parser_parse_didl (parser,
+                                                        didl_xml,
+                                                        &error)) {
                         g_warning ("%s\n", error->message);
                         g_error_free (error);
                 }
 
+                g_object_unref (parser);
                 g_free (didl_xml);
 
                 data->starting_index += number_returned;
@@ -748,6 +768,8 @@ browse_cb (GUPnPServiceProxy       *content_dir,
                                 data->id,
                                 data->starting_index,
                                 MIN (remaining, MAX_BROWSE));
+                else
+                        update_container_child_count (content_dir, data->id);
         } else if (error) {
                 GUPnPServiceInfo *info;
 
@@ -788,7 +810,7 @@ browse (GUPnPServiceProxy *content_dir,
                                  "BrowseDirectChildren",
                                  "Filter",
                                  G_TYPE_STRING,
-                                 "*",
+                                 "@childCount",
                                  "StartingIndex",
                                  G_TYPE_UINT,
                                  starting_index,
@@ -803,8 +825,8 @@ browse (GUPnPServiceProxy *content_dir,
 
 static void
 append_media_server (GUPnPDeviceProxy *proxy,
-                     GtkTreeModel          *model,
-                     GtkTreeIter           *parent_iter)
+                     GtkTreeModel     *model,
+                     GtkTreeIter      *parent_iter)
 {
         GUPnPDeviceInfo   *info;
         GUPnPServiceProxy *content_dir;
