@@ -49,6 +49,8 @@ typedef struct {
 static void
 get_icon_url_data_free (GetIconURLData *data)
 {
+        g_object_unref (data->info);
+
         g_free (data->mime_type);
         g_slice_free (GetIconURLData, data);
 }
@@ -157,7 +159,7 @@ schedule_icon_update (GUPnPDeviceInfo            *info,
                 return;
         }
 
-        data->info = info;
+        data->info = g_object_ref (info);
         data->callback = callback;
 
         pending_gets = g_list_prepend (pending_gets, data);
@@ -172,12 +174,14 @@ schedule_icon_update (GUPnPDeviceInfo            *info,
 void
 unschedule_icon_update (GUPnPDeviceInfo *info)
 {
-        while (pending_gets) {
+        GList *gets;
+
+        for (gets = pending_gets; gets; gets = gets->next) {
                 GetIconURLData *data;
                 const char *udn1;
                 const char *udn2;
 
-                data = pending_gets->data;
+                data = gets->data;
                 udn1 = gupnp_device_info_get_udn (info);
                 udn2 = gupnp_device_info_get_udn (data->info);
 
