@@ -37,37 +37,90 @@
 #define SWITCH_SERVICE "urn:schemas-upnp-org:service:SwitchPower:1"
 #define NETWORK_LIGHT "urn:schemas-upnp-org:device:DimmableLight:1"
 
-static GUPnPContext     *context;
-static GUPnPRootDevice  *dev;
-static GUPnPServiceInfo *switch_power;
-static GUPnPServiceInfo *dimming;
+typedef struct
+{
+        GUPnPRootDevice *dev;
+        GUPnPServiceInfo *switch_power;
+        GUPnPServiceInfo *dimming;
+} NetworkLight;
+
+static GUPnPContextManager *context_manager;
+static GHashTable *nl_hash;
 
-static GUPnPControlPoint *cp;
 /* Other network light services on the network */
 static GList *switch_proxies;
 static GList *dimming_proxies;
 
-static char *uuid;
+static GUPnPXMLDoc *doc;
 static char *desc_location;
+static char uuid[37];
+
+static NetworkLight *
+network_light_new (GUPnPRootDevice  *dev,
+                   GUPnPServiceInfo *switch_power,
+                   GUPnPServiceInfo *dimming)
+{
+        NetworkLight *network_light;
+
+        network_light = g_slice_new (NetworkLight);
+
+        network_light->dev = dev;
+        network_light->switch_power = switch_power;
+        network_light->dimming = dimming;
+
+        return network_light;
+}
+
+static void
+network_light_free (NetworkLight *network_light)
+{
+        g_object_unref (network_light->dev);
+        g_object_unref (network_light->switch_power);
+        g_object_unref (network_light->dimming);
+
+        g_slice_free (NetworkLight, network_light);
+}
 
 void
 notify_status_change (gboolean status)
 {
-        gupnp_service_notify (GUPNP_SERVICE (switch_power),
-                              "Status",
-                              G_TYPE_BOOLEAN,
-                              status,
-                              NULL);
+        GList *network_lights;
+        GList *nl_node;
+
+        network_lights = g_hash_table_get_values (nl_hash);
+
+        for (nl_node = network_lights;
+             nl_node != NULL;
+             nl_node = nl_node->next) {
+                NetworkLight *nl = (NetworkLight *) nl_node->data;
+
+                gupnp_service_notify (GUPNP_SERVICE (nl->switch_power),
+                                      "Status",
+                                      G_TYPE_BOOLEAN,
+                                      status,
+                                      NULL);
+        }
 }
 
 void
 notify_load_level_change (gint load_level)
 {
-        gupnp_service_notify (GUPNP_SERVICE (dimming),
-                              "LoadLevelStatus",
-                              G_TYPE_UINT,
-                              load_level,
-                              NULL);
+        GList *network_lights;
+        GList *nl_node;
+
+        network_lights = g_hash_table_get_values (nl_hash);
+
+        for (nl_node = network_lights;
+             nl_node != NULL;
+             nl_node = nl_node->next) {
+                NetworkLight *nl = (NetworkLight *) nl_node->data;
+
+                gupnp_service_notify (GUPNP_SERVICE (nl->dimming),
+                                      "LoadLevelStatus",
+                                      G_TYPE_UINT,
+                                      load_level,
+                                      NULL);
+        }
 }
 
 void
@@ -259,14 +312,16 @@ xml_util_get_element (xmlNode *node,
         return node;
 }
 
-static char *
-change_uuid (xmlDoc *doc)
+static void init_uuid ()
 {
+        uuid_t uuid_context;
         xmlNode *uuid_node;
-        uuid_t uuid;
-        char *udn, uuidstr[37];
+        char *udn;
 
-        uuid_node = xml_util_get_element ((xmlNode *) doc,
+        uuid_generate (uuid_context);
+        uuid_unparse (uuid_context, uuid);
+
+        uuid_node = xml_util_get_element ((xmlNode *) doc->doc,
                                           "root",
                                           "device",
                                           "UDN",
@@ -275,24 +330,14 @@ change_uuid (xmlDoc *doc)
                 g_critical ("Failed to find UDN element"
                             "in device description");
 
-                return NULL;
+                return;
         }
 
-        uuid_generate (uuid);
-        uuid_unparse (uuid, uuidstr);
-
-        if (uuidstr == NULL) {
-                udn = (char *) xmlNodeGetContent (uuid_node);
-                g_warning ("Failed to generate UUID for device description."
-                           "Using default UDN: %s", udn);
-                udn = g_strdup (udn);
-        } else
-                udn = g_strdup_printf ("uuid:%s", uuidstr);
+        udn = g_strdup_printf ("uuid:%s", uuid);
 
         xmlNodeSetContent (uuid_node, (unsigned char *) udn);
-        g_free (udn);
 
-        return g_strdup (uuidstr);
+        g_free (udn);
 }
 
 void on_service_proxy_action_ret (GUPnPServiceProxy *proxy,
@@ -445,60 +490,18 @@ on_network_light_unavailable (GUPnPControlPoint *control_point,
 static gboolean
 init_server (GUPnPContext *context)
 {
-        GError *error;
-        xmlDoc *doc = NULL;
-        char *ext_desc_path;
-
-        g_print ("Running on port %d\n", gupnp_context_get_port (context));
-
-        doc = xmlParseFile (DATA_DIR "/" DESCRIPTION_DOC);
-        if (doc == NULL) {
-                g_critical ("Unable to load the XML description file %s",
-                            DESCRIPTION_DOC);
-                g_object_unref (context);
-                return FALSE;
-        }
-
-        /* Geting a new uuid and updating the xmlDoc */
-        uuid = change_uuid (doc);
-        if (!uuid) {
-                xmlFreeDoc (doc);
-                g_object_unref (context);
-
-                return FALSE;
-        }
-
-        /* saving the xml file to the temporal location with the uuid name */
-        desc_location = g_strdup_printf ("%s/gupnp-network-light-%s.xml",
-                                         g_get_tmp_dir (),
-                                         uuid);
-        g_assert (desc_location != NULL);
-
-        if (xmlSaveFile (desc_location, doc) < 0) {
-                g_print ("Error saving description file to %s.\n",
-                         desc_location);
-
-                g_free (desc_location);
-                xmlFreeDoc (doc);
-                g_object_unref (context);
-
-                return FALSE;
-        }
-
-        ext_desc_path = g_strdup_printf ("/%s.xml",uuid);
-
-        gupnp_context_host_path (context, desc_location, ext_desc_path);
-        gupnp_context_host_path (context, DATA_DIR, "");
+        NetworkLight *network_light;
+        GUPnPRootDevice *dev;
+        GUPnPServiceInfo *switch_power;
+        GUPnPServiceInfo *dimming;
+        GError *error = NULL;
 
         /* Create root device */
         dev = gupnp_root_device_new_full (context,
                                           gupnp_resource_factory_get_default (),
                                           doc,
-                                          ext_desc_path);
-        g_free (ext_desc_path);
-
-        /* Free doc when root device is destroyed */
-        g_object_weak_ref (G_OBJECT (dev), (GWeakNotify) xmlFreeDoc, doc);
+                                          desc_location,
+                                          DATA_DIR);
 
         switch_power = gupnp_device_info_get_service (GUPNP_DEVICE_INFO (dev),
                                                       SWITCH_SERVICE);
@@ -529,18 +532,64 @@ init_server (GUPnPContext *context)
                                   NULL);
         }
 
+        network_light = network_light_new (dev, switch_power, dimming);
+        g_hash_table_insert (nl_hash, g_object_ref (context), network_light);
+
         /* Run */
         gupnp_root_device_set_available (dev, TRUE);
 
+        g_print ("Attaching to IP/Host %s on port %d\n",
+                 gupnp_context_get_host_ip (context),
+                 gupnp_context_get_port (context));
+
+        return TRUE;
+}
+
+static gboolean
+prepare_desc ()
+{
+        GError *error;
+
+        error = NULL;
+        doc = gupnp_xml_doc_new_from_path (DATA_DIR "/" DESCRIPTION_DOC,
+                                           &error);
+        if (doc == NULL) {
+                g_critical ("Unable to load the XML description file: %s",
+                            error->message);
+
+                g_error_free (error);
+
+                return FALSE;
+        }
+
+        /* create and set the UUID in the xmlDoc */
+        init_uuid ();
+
+        /* saving the xml file to the temporal location with the uuid name */
+        desc_location = g_strdup_printf ("%s/gupnp-network-light-%s.xml",
+                                         g_get_tmp_dir (),
+                                         uuid);
+        g_assert (desc_location != NULL);
+
+        if (xmlSaveFile (desc_location, doc->doc) < 0) {
+                g_print ("Error saving description file to %s.\n",
+                         desc_location);
+
+                g_free (desc_location);
+                g_object_unref (doc);
+
+                return FALSE;
+        }
+
         return TRUE;
 }
 
 static gboolean
 init_client (GUPnPContext *context)
 {
+        GUPnPControlPoint *cp;
+
         cp = gupnp_control_point_new (context, NETWORK_LIGHT);
-        switch_proxies = NULL;
-        dimming_proxies = NULL;
 
         g_signal_connect (cp,
                           "device-proxy-available",
@@ -553,61 +602,93 @@ init_client (GUPnPContext *context)
 
         gssdp_resource_browser_set_active (GSSDP_RESOURCE_BROWSER (cp), TRUE);
 
+        /* Let context manager take care of the control point life cycle */
+        gupnp_context_manager_manage_control_point (context_manager, cp);
+
+        /* We don't need to keep our own references to the control points */
+        g_object_unref (cp);
+
         return TRUE;
 }
 
+static void
+on_context_available (GUPnPContextManager *context_manager,
+                      GUPnPContext        *context,
+                      gpointer             user_data)
+{
+        /* Initialize client-side stuff */
+        init_client (context);
+
+        /* Then the server-side stuff */
+        init_server (context);
+}
+
+static void
+on_context_unavailable (GUPnPContextManager *context_manager,
+                        GUPnPContext        *context,
+                        gpointer             user_data)
+{
+        g_print ("Dettaching from IP/Host %s and port %d\n",
+                 gupnp_context_get_host_ip (context),
+                 gupnp_context_get_port (context));
+
+        g_hash_table_remove (nl_hash, context);
+}
+
+static gboolean
+context_equal (GUPnPContext *context1, GUPnPContext *context2)
+{
+        return g_ascii_strcasecmp (gupnp_context_get_host_ip (context1),
+                                   gupnp_context_get_host_ip (context2)) == 0;
+}
+
 gboolean
 init_upnp (void)
 {
         GError *error = NULL;
 
-        context = gupnp_context_new (NULL, NULL, 0, &error);
+        switch_proxies = NULL;
+        dimming_proxies = NULL;
 
-        if (error) {
-                g_error (error->message);
-                g_error_free (error);
+        nl_hash = g_hash_table_new_full (g_direct_hash,
+                                         (GEqualFunc) context_equal,
+                                         g_object_unref,
+                                         (GDestroyNotify) network_light_free);
 
+        if (!prepare_desc (&error)) {
                 return FALSE;
         }
 
-        /* Initialize client-side stuff */
-        if (!init_client (context))
-                return FALSE;
+        context_manager = gupnp_context_manager_new (NULL, 0);
+        g_assert (context_manager != NULL);
 
-        /* Then the server-side stuff */
-        if (!init_server(context))
-                return FALSE;
+        g_signal_connect (context_manager,
+                          "context-available",
+                          G_CALLBACK (on_context_available),
+                          NULL);
+        g_signal_connect (context_manager,
+                          "context-unavailable",
+                          G_CALLBACK (on_context_unavailable),
+                          NULL);
 
         return TRUE;
 }
 
-static void
-deinit_server (void)
-{
-        g_object_unref (switch_power);
-        g_object_unref (dimming);
-        g_object_unref (dev);
-        g_object_unref (context);
-
-	if (g_remove (desc_location) != 0)
-                g_warning ("error removing %s\n", desc_location);
-
-        g_free (desc_location);
-        g_free (uuid);
-}
-
-static void
-deinit_client (void)
-{
-        g_object_unref (cp);
-        g_list_foreach (switch_proxies, (GFunc) g_object_unref, NULL);
-        g_list_foreach (dimming_proxies, (GFunc) g_object_unref, NULL);
-}
-
 void
 deinit_upnp (void)
 {
-        deinit_server ();
-        deinit_client ();
+        g_object_unref (context_manager);
+
+        g_hash_table_unref (nl_hash);
+
+        g_list_foreach (switch_proxies, (GFunc) g_object_unref, NULL);
+        g_list_foreach (dimming_proxies, (GFunc) g_object_unref, NULL);
+
+        /* Unref the descriptiont doc */
+        g_object_unref (doc);
+
+        if (g_remove (desc_location) != 0)
+                g_warning ("error removing %s\n", desc_location);
+        g_free (desc_location);
 }
 
