@@ -1,7 +1,7 @@
 /*
- * Copyright (C) 2007 Zeeshan Ali <zeenix@gstreamer.net>
+ * Copyright (C) 2007 Zeeshan Ali (Khattak) <zeeshanak@gnome.org>
  *
- * Authors: Zeeshan Ali <zeenix@gstreamer.net>
+ * Authors: Zeeshan Ali (Khattak) <zeeshanak@gnome.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -682,7 +682,7 @@ on_action_complete (GUPnPServiceProxy       *proxy,
 
                 error_dialog =
                         gtk_message_dialog_new (
-                                        dialog,
+                                        GTK_WINDOW (dialog),
                                         GTK_DIALOG_MODAL,
                                         GTK_MESSAGE_ERROR,
                                         GTK_BUTTONS_CLOSE,
@@ -690,7 +690,7 @@ on_action_complete (GUPnPServiceProxy       *proxy,
                                         error->code,
                                         error->message);
 
-                gtk_dialog_run (error_dialog);
+                gtk_dialog_run (GTK_DIALOG (error_dialog));
                 gtk_widget_destroy (error_dialog);
 
                 g_error_free (error);
@@ -730,7 +730,7 @@ on_action_invocation (GtkButton *button,
 }
 
 void
-init_action_dialog (GladeXML *glade_xml)
+init_action_dialog (GtkBuilder *builder)
 {
         GtkWidget *image;
 
@@ -742,44 +742,53 @@ init_action_dialog (GladeXML *glade_xml)
                 gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
 
         /* Dialog box and tables */
-        dialog = glade_xml_get_widget (glade_xml, "action-invocation-dialog");
+        dialog = GTK_WIDGET (gtk_builder_get_object (
+                                        builder,
+                                        "action-invocation-dialog"));
         g_assert (dialog != NULL);
-        in_args_table = glade_xml_get_widget (glade_xml,
-                                              "in-action-arguments-table");
+        in_args_table = GTK_WIDGET (gtk_builder_get_object (
+                                        builder,
+                                        "in-action-arguments-table"));
         g_assert (in_args_table != NULL);
-        out_args_table = glade_xml_get_widget (glade_xml,
-                                               "out-action-arguments-table");
+        out_args_table = GTK_WIDGET (gtk_builder_get_object (
+                                        builder,
+                                        "out-action-arguments-table"));
         g_assert (out_args_table != NULL);
 
         /* All the labels */
-        in_args_label = glade_xml_get_widget (glade_xml,
-                                              "in-action-arguments-label");
+        in_args_label = GTK_WIDGET (gtk_builder_get_object (
+                                        builder,
+                                        "in-action-arguments-label"));
         g_assert (in_args_label != NULL);
-        out_args_label = glade_xml_get_widget (glade_xml,
-                                              "out-action-arguments-label");
+        out_args_label = GTK_WIDGET (gtk_builder_get_object (
+                                        builder,
+                                        "out-action-arguments-label"));
         g_assert (out_args_label != NULL);
-        device_label = glade_xml_get_widget (glade_xml, "device-label");
+        device_label = GTK_WIDGET (gtk_builder_get_object (builder,
+                                                           "device-label"));
         g_assert (device_label != NULL);
         gtk_size_group_add_widget (static_labels_group, device_label);
-        service_label = glade_xml_get_widget (glade_xml, "service-label");
+        service_label = GTK_WIDGET (gtk_builder_get_object (builder,
+                                                            "service-label"));
         g_assert (service_label != NULL);
         gtk_size_group_add_widget (static_labels_group, service_label);
-        action_label = glade_xml_get_widget (glade_xml, "action-label");
+        action_label = GTK_WIDGET (gtk_builder_get_object (builder,
+                                                           "action-label"));
         g_assert (action_label != NULL);
         gtk_size_group_add_widget (static_labels_group, action_label);
 
         /* the images */
-        image = glade_xml_get_widget (glade_xml, "device-image");
+        image = GTK_WIDGET (gtk_builder_get_object (builder, "device-image"));
         g_assert (image != NULL);
         gtk_image_set_from_pixbuf (GTK_IMAGE (image),
                                    get_icon_by_id (ICON_DEVICE));
 
-        image = glade_xml_get_widget (glade_xml, "service-image");
+        image = GTK_WIDGET (gtk_builder_get_object (builder, "service-image"));
         g_assert (image != NULL);
         gtk_image_set_from_pixbuf (GTK_IMAGE (image),
                                    get_icon_by_id (ICON_SERVICE));
 
-        image = glade_xml_get_widget (glade_xml, "action-image");
+        image = GTK_WIDGET (gtk_builder_get_object (builder, "action-image"));
         g_assert (image != NULL);
         gtk_image_set_from_pixbuf (GTK_IMAGE (image),
                                    get_icon_by_id (ICON_ACTION));
